/**************************************************************

   Program Title:  GenY32 GRBL Port
   
   Program Name:  main.c	 Version: 1.02b
   
   Author:  ALD	  			 Date:  May 10, 2014
   
   Compiled with AVR Studio 6.2.993
   
   This file contains routines that implement the basic initialization 
   for the SOC Robotics GenY32 initialization and GRBL instantiation. 
   
   Copyright 2014, Tony da Costa  All rights reserved.
   
   Supported Features:
   - step engine pulse and duration is interrupt driven.
   - 32bit processor (AT32UC3C2512) runs at 64MHz, hardware FPU, 512KFlash, 64K SRAM
   - 2 interrupt driven serial ports available
   - high speed A/D and D/A
   - DMA engine
   - uSD support and simple file management routines
   - parameters stored in upper flash memory (simulates EEPROM)
   
   V1.01, V1.02, V1.02b changes made by Stephen Swift, SOC Robotics, Inc to support several new features.

   SOC Robotics has added the following enhancements:
   
   Version 1.01 Changes
     - printf statement is now supported for output to USB
	 - printf0/scanf0 support for USART0 (joystick/pendant serial port) - interrupt driven
	 - printf3/scanf3 support for USART3 (general purpose) - interrupt driven
	 - Change prompt character: M300 - $, M301 - OK
	 - Tandem drive (link X axis with A axis)  - M302 - enable,  M303 - disable
	 - Software initiated DFU bootloader start - M202
	 - Simulate 3D printer temperature output - M105 - test for Android tablet integration
	 - CJ34B Joystick beta support - M102 - print joystick position, M103 - don't print joystick position
	 - Print joystick analog values - M100
	 - Print system position - M101

   Version 1.02 Changes
      - 4 Axis support (beta version - rotary not supported - assumed linear - code partially extracted from openpnp code)
	  - Timed asymmetric coolant control using 1msec interrupt - M306 Ox.xxx Uy.yyy, M307 - disable FAN1
		       where O is on time in seconds and U is off time	 M308 Ox.xxx Uy.yyy, M309 - disable FAN1

   Version 1.02b Changes
	 - Spindle speed command now enables analog output on OUT2, spindle direction shifted to OUT3
	 - M211 enables spindle analog output and M214oxxxx set spindle speed
	  
   TBD
	 - Autoload configuration information from uSD
	 - New M commands to support more Digital/Analog IO
	 - File management and uSD G Code execution
	 - Hardware PWM Spindle control on FAN1 and FAN2
	 - Integrate ESP8266 Serial WiFi module
	 - Integrate PicoM with with wireless RF24L01 support
	 
   Other features coming
	 - 5 and 6 Axis support
	 - Smart Limit Switch support
	 - Optimized interrupt support for step pulse off
	 
   Copyright 2015, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2014, Tony da Costa  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TONY DA COSTA AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/*
 * Copyright (C) 2006-2014 by SOC Robotics, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */
/*
 * Include header files for all drivers that have been imported from
 * Atmel Software Framework (ASF).
 */
#include <asf.h>
#include <stdio.h>
#include <string.h>

// Variables to support new SOC Robotics GenY32 features
char promptchar,tandemmode,pwm1mode=DISABLE,pwm1state=OFF,pwm2mode=DISABLE,pwm2state=OFF,pwmactivation=0,pwm1hardware=DISABLE,pwm2hardware=DISABLE;
int countdown1,countdown2,countdown3,pwm1oncnt=0,pwm1offcnt=0,pwm2oncnt=0,pwm2offcnt=0;
float pwmontime=0.0,pwmofftime=0.0,pwm1ontime=0.0,pwm1offtime=0.0,pwm2ontime=0.0,pwm2offtime=0.0;
float pwm1period=0.0,pwm2period=0.0,pwm1dutycycle,pwm2dutycycle;

//Connect USB to stdout
static int uart_put(void volatile *stream, char c)
{
	//ser_PutChar(COM1,c);
	ser_PutChar(COMUSB,c);
	return 0;
}
int (*ptr_put)(void volatile*, char)=&uart_put;

extern int maingrbl(void);


int main (void)
{
	
	sysclk_init();
	board_init();
	irq_initialize_vectors();
	sleepmgr_init();
	wdt_disable();
	timer_Init();
	ser_Init();
	nvram_Init();
	
	// Turn on interrupts
	cpu_irq_enable();

	//Set unbuffered output to stdout
	setbuf(stdout, NULL);
	setbuf(stdin, NULL);

	// Check to see if CJ34b present
	Check_CJ34B_Present();
	
	while(1) //Shouldn't need a loop as maingrbl shouldn't return, but just in case...
	{
		maingrbl();
	}

	return 0; //Should never get here
}
